package org.artifactory.ph;

import org.artifactory.common.repo.PackageType;
import org.artifactory.common.repo.RepoInfo;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.exception.PackageRuntimeException;
import org.artifactory.core.api.ph.service.PackageSpecificationAdapter;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.cache.expirable.IndexChecker;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.artifactory.ph.PackageRepoTypeMapper.packageType;

public class PackageSpecificationImpl extends IndexChecker implements PackageSpecification {
    private static final Logger log = LoggerFactory.getLogger(PackageSpecificationImpl.class);

    private final PackageSpecificationAdapter packageSpecificationAdapter;

    public PackageSpecificationImpl(PackageSpecificationAdapter packageSpecificationAdapter) {
        this.packageSpecificationAdapter = packageSpecificationAdapter;
    }

    @Override
    public boolean isExpirable(RepoType repoType, String repoKey, @NotNull String path) {
        try {
            PackageType packageType = packageType(repoType);
            if (packageType == null || !packageSpecificationAdapter.isSupportedPackage(packageType)) {
                return false;
            }
            return packageSpecificationAdapter.isExpirable(packageType, path);
        } catch (PackageException e) {
            log.error("Failed to check if artifact is expired, reason:{}", e.getInternalReason());
            throw new PackageRuntimeException(e.getStatus(), e.getPublicReason(), e.getInternalReason(), e);
        }
    }

    @Override
    public boolean isLocalGenerated(@NotNull RepoType repoType, @NotNull String repoKey, @NotNull String path) {
        try {
            PackageType packageType = packageType(repoType);
            if (packageType == null || !packageSpecificationAdapter.isSupportedPackage(packageType)) {
                return false;
            }
            return packageSpecificationAdapter.isLocalGenerated(packageType, path);
        } catch (PackageException e) {
            log.error("Failed to check if artifact is locally generated, reason:{}", e.getInternalReason());
            throw new PackageRuntimeException(e.getStatus(), e.getPublicReason(), e.getInternalReason(), e);
        }
    }

    @Override
    public boolean shouldExcludePropertyFromMirroring(@NotNull RepoType repoType, @NotNull String propertyKey) {
        try {
            PackageType packageType = packageType(repoType);
            if (packageType == null || !packageSpecificationAdapter.isSupportedPackage(packageType)) {
                return false;
            }
            return packageSpecificationAdapter.isLocallyGeneratedProperty(packageType, propertyKey);
        } catch (PackageException e) {
            log.error("Failed to check if artifact is locally generated, reason:{}", e.getInternalReason());
            throw new PackageRuntimeException(e.getStatus(), e.getPublicReason(), e.getInternalReason(), e);
        }
    }

    @Override
    public boolean isPackageArtifact(@NotNull String repoKey, @NotNull String path) {
        try {
            if (!packageSpecificationAdapter.isSupportedPackage(repoKey)) {
                return false;
            }
            return packageSpecificationAdapter.isPackageArtifact(repoKey, path);
        } catch (PackageException e) {
            log.error("Failed to check if artifact is locally generated, reason:{}", e.getInternalReason());
            throw new PackageRuntimeException(e.getStatus(), e.getPublicReason(), e.getInternalReason(), e);
        }
    }

    @Override
    public boolean hasReindex(org.artifactory.common.repo.RepoType repoType, RepoType packageType) {
        return packageSpecificationAdapter.hasReindex(getRepoInfo(repoType, packageType));
    }

    private RepoInfo getRepoInfo(org.artifactory.common.repo.RepoType repoType, RepoType packageType) {
        PackageType packageT = PackageRepoTypeMapper.packageType(packageType);
        switch (repoType) {
            case REMOTE:
                return new RepoInfo(org.artifactory.common.repo.RepoType.REMOTE, packageT);
            case VIRTUAL:
                return new RepoInfo(org.artifactory.common.repo.RepoType.VIRTUAL, packageT);
            case LOCAL:
            default:
                return new RepoInfo(org.artifactory.common.repo.RepoType.LOCAL, packageT);
        }
    }


}
