package org.artifactory.ph;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.core.api.ph.exception.PackageRepoDescriptorException;
import org.artifactory.core.api.ph.service.PackageArtifactoryConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.repo.Repo;
import org.artifactory.repo.service.InternalRepositoryService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.mock.web.MockHttpServletRequest;

import java.util.HashMap;
import java.util.Map;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;
import static org.testng.Assert.assertEquals;

/**
 * @author Ori Yafe
 */
public class PackageArtifactoryConfigServiceImplTest {

    @Mock
    private InternalRepositoryService repositoryService;
    @Mock
    private CentralConfigService centralConfigService;
    @Mock
    private ArtifactoryContext artifactoryContext;
    @Mock
    CentralConfigDescriptor centralConfigDescriptor;
    @Mock
    ArtifactorySystemProperties artifactorySystemProperties;

    private PackageArtifactoryConfigService packageArtifactoryConfigService;

    @BeforeEach
    private void setUp() {
        openMocks(this);

        ArtifactoryHome home = Mockito.mock(ArtifactoryHome.class);
        when(home.getArtifactoryProperties()).thenReturn(artifactorySystemProperties);
        ArtifactoryHome.bind(home);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);

        packageArtifactoryConfigService = new PackageArtifactoryConfigServiceImpl(repositoryService,
                centralConfigService);

    }

    @Test
    public void testGetStringFromConfig() throws PackageRepoDescriptorException {
        String repoKey = "local";
        prepareTest(repoKey);
        // no such field
        Assertions.assertThrows(
                PackageRepoDescriptorException.class, ()->
                        packageArtifactoryConfigService.getStringFromRepoConfig(repoKey, "nosuchmethod", "default"),
                "default");

        Assertions.assertEquals(packageArtifactoryConfigService.getStringFromRepoConfig(repoKey,
                        "getTerraformType", "default"),
                "MODULE");
    }

    @Test
    public void testGetIntegerFromConfig() throws PackageRepoDescriptorException {
        String repoKey = "local";
        prepareTest(repoKey);
        // no such method
        Assertions.assertThrows(PackageRepoDescriptorException.class, ()->
                packageArtifactoryConfigService.getIntFromRepoConfig(repoKey, "nosuchmethod", 33));
        // casting error
        Assertions.assertThrows(PackageRepoDescriptorException.class, ()->
                packageArtifactoryConfigService.getIntFromRepoConfig(repoKey, "isForceNugetAuthentication", 33));

        Assertions.assertEquals(
                packageArtifactoryConfigService.getIntFromRepoConfig(repoKey, "getDockerTagRetention", 33), 1);
    }

    @Test
    public void testGetBooleanFromConfig() throws PackageRepoDescriptorException {
        String repoKey = "local";
        prepareTest(repoKey);
        // no such method
        Assertions.assertThrows(PackageRepoDescriptorException.class, ()->
                packageArtifactoryConfigService.getBooleanFromRepoConfig(repoKey, "nosuchmethod", true));

        // casting error
        Assertions.assertThrows(PackageRepoDescriptorException.class, ()->
                packageArtifactoryConfigService.getBooleanFromRepoConfig(repoKey, "getDockerTagRetention", true));

        Assertions.assertFalse(packageArtifactoryConfigService.getBooleanFromRepoConfig(repoKey,
                "isDebianTrivialLayout", false));
    }

    @Test
    public void testGetArtifactoryBaseUrlFromHttpUrl() {
        when(artifactoryContext.getCentralConfig()).thenReturn(centralConfigService);
        when(centralConfigService.getDescriptor()).thenReturn(centralConfigDescriptor);
        String requestUrl = "https://lala.land.com";
        when(centralConfigDescriptor.getUrlBase()).thenReturn(null);
        MockHttpServletRequest mockRequest = new MockHttpServletRequest("GET", requestUrl);
        mockRequest.setServerName("lala.land.com");
        mockRequest.setScheme("https");
        mockRequest.setServerPort(443);
        String servletContextUrl = packageArtifactoryConfigService.getArtifactoryBaseUrl(mockRequest);
        assertEquals(servletContextUrl, requestUrl);
    }

    @Test
    public void testGetArtifactoryBaseUrlFromDescriptorWithoutContextPath() {
        when(artifactoryContext.getCentralConfig()).thenReturn(centralConfigService);
        when(centralConfigService.getDescriptor()).thenReturn(centralConfigDescriptor);
        String contextPath = "/artifactory";
        String requestUrl = "https://lala.land.com";
        when(artifactorySystemProperties.getProperty(ConstantValues.contextPath)).thenReturn(contextPath);
        when(centralConfigDescriptor.getUrlBase()).thenReturn(requestUrl);
        MockHttpServletRequest mockRequest = new MockHttpServletRequest("GET", requestUrl);
        String servletContextUrl = packageArtifactoryConfigService.getArtifactoryBaseUrl(mockRequest);
        assertEquals(servletContextUrl, requestUrl + contextPath);
    }

    @Test
    public void testGetArtifactoryBaseUrlFromDescriptorWithContextPath() {
        when(artifactoryContext.getCentralConfig()).thenReturn(centralConfigService);
        when(centralConfigService.getDescriptor()).thenReturn(centralConfigDescriptor);
        String requestUrl = "https://lala.land.com/artifactory";
        when(centralConfigDescriptor.getUrlBase()).thenReturn(requestUrl);
        MockHttpServletRequest mockRequest = new MockHttpServletRequest("GET", requestUrl);
        String servletContextUrl = packageArtifactoryConfigService.getArtifactoryBaseUrl(mockRequest);
        assertEquals(servletContextUrl, requestUrl);
    }


    private void prepareTest(String repoKey) {
        Repo repo = mock(Repo.class);
        when(repo.getKey()).thenReturn(repoKey);
        when(repositoryService.repositoryByKey(repoKey)).thenReturn(repo);
        when(repo.isReal()).thenReturn(true);
        when(repo.isLocal()).thenReturn(true);
        when(centralConfigService.getDescriptor()).thenReturn(centralConfigDescriptor);
        LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        Map<String, LocalRepoDescriptor> localRepoDescriptorMap = new HashMap<>();
        localRepoDescriptorMap.put(repoKey, localRepoDescriptor);
        when(centralConfigDescriptor.getLocalAndFederatedRepositoriesMap()).thenReturn(localRepoDescriptorMap);
    }




}
