/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.actions;

import com.google.common.collect.Lists;
import org.artifactory.addon.AddonType;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.watch.ArtifactWatchAddon;
import org.artifactory.api.build.BuildService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.download.FolderDownloadConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RemoteRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.fs.WatchersInfo;
import org.artifactory.mime.NamingUtils;
import org.artifactory.ph.PackageSpecification;
import org.artifactory.repo.RepoPath;
import org.jfrog.client.util.PathUtils;
import org.jfrog.security.util.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static org.artifactory.rest.common.util.RestUtils.calcRepoPathFromRequestPath;

/**
 * Builds an actions list to be added to a tree node
 *
 * @author Shay Yaakov
 */
@Component
public class TreeNodeActionsPopulator {

    private static final String REMOTE_REPO = "remote";
    private static final String VIRTUAL_REPO = "virtual";
    private static final String RELEASE_BUNDLES_REPO = "releaseBundles";
    private static final String SUPPORT_BUNDLES_REPO = "supportBundles";
    private static final String TRASH = "trash";

    private final RepositoryService repoService;
    private final AuthorizationService authService;
    private final CentralConfigService centralConfig;
    private final BuildService buildService;
    private final AddonsManager addonsManager;
    private final PackageSpecification packageSpecification;

    @Autowired
    public TreeNodeActionsPopulator(RepositoryService repoService, AuthorizationService authService,
            CentralConfigService centralConfig, BuildService buildService, AddonsManager addonsManager,
            PackageSpecification packageSpecification) {
        this.repoService = repoService;
        this.authService = authService;
        this.centralConfig = centralConfig;
        this.buildService = buildService;
        this.addonsManager = addonsManager;
        this.packageSpecification = packageSpecification;
    }

    public void populateForRepository(TabsAndActions tabsAndActions) {
        List<TabOrAction> actions = Lists.newArrayList();
        RepoPath repoPath = tabsAndActions.getRepoPath();
        String repoType = tabsAndActions.getRepoType();
        if (REMOTE_REPO.equals(repoType) || VIRTUAL_REPO.equals(repoType)) {
            actions.add(refresh());
            actions.add(virtualZapCaches(tabsAndActions.getRepoType()));
            actions.add(remoteVirtualReindex(tabsAndActions.getRepoPkgType(), tabsAndActions.getRepoType()));
            actions.add(favorites());
        } else if (RELEASE_BUNDLES_REPO.equals(repoType)) {
            actions.add(refresh());
            actions.add(watch(repoPath));
            actions.add(deleteContent(repoPath));
            actions.add(favorites());
        } else if (SUPPORT_BUNDLES_REPO.equals(repoType)) {
            actions.add(refresh());
            actions.add(downloadFolder(repoPath));
            actions.add(deleteContent(repoPath));
        } else if (TRASH.equals(repoType)) {
            actions.add(refresh());
            actions.add(searchTrash());
            actions.add(emptyTrash());
        } else {
            actions.add(downloadFolder(repoPath));
            actions.add(refresh());
            actions.add(copyContent(repoPath));
            actions.add(moveContent(repoPath));
            actions.add(watch(repoPath));
            actions.add(zap(repoPath));
            actions.add(reindex(repoPath, tabsAndActions.getRepoPkgType(), tabsAndActions.getRepoType()));
            actions.add(deleteVersions(repoPath));
            actions.add(deleteContent(repoPath));
            actions.add(favorites());
        }
        tabsAndActions.setActions(actions.stream().filter(Objects::nonNull).collect(Collectors.toList()));
    }

    public void populateForFolder(TabsAndActions tabsAndActions) {
        List<TabOrAction> actions = Lists.newArrayList();
        RepoPath repoPath = tabsAndActions.getRepoPath();
        String repoType = tabsAndActions.getRepoType();
        if (REMOTE_REPO.equals(repoType) || VIRTUAL_REPO.equals(repoType)) {
            actions.add(refresh());
        } else if (RELEASE_BUNDLES_REPO.equals(repoType)) {
            actions.add(refresh());
            actions.add(watch(repoPath));
            actions.add(downloadFolder(repoPath));
        } else if (SUPPORT_BUNDLES_REPO.equals(repoType)) {
            actions.add(refresh());
            actions.add(downloadFolder(repoPath));
            actions.add(delete(repoPath));
        } else if (TRASH.equals(repoType)) {
            actions.add(refresh());
            actions.add(restore());
            actions.add(deletePermanently());
        } else {
            actions.add(downloadFolder(repoPath));
            actions.add(refresh());
            actions.add(copy(repoPath));
            actions.add(move(repoPath));
            actions.add(watch(repoPath));
            actions.add(zap(repoPath));
            actions.add(deleteVersions(repoPath));
            actions.add(delete(repoPath));
        }
        tabsAndActions.setActions(actions.stream()
                .filter(Objects::nonNull)
                .collect(Collectors.toList()));
    }

    public void populateForFile(TabsAndActions tabsAndActions) {
        List<TabOrAction> actions = Lists.newArrayList();
        RepoPath repoPath = tabsAndActions.getRepoPath();
        String repoType = tabsAndActions.getRepoType();
        boolean isCached = tabsAndActions.getCached();
        if (REMOTE_REPO.equals(repoType)) {
            if (userCanDownloadFromRemote(tabsAndActions)) {
                actions.add(download());
            }
        } else if (VIRTUAL_REPO.equals(repoType)) {
            if (!isCached) {
                actions.add(download());
                tabsAndActions.setActions(actions.stream().filter(Objects::nonNull).collect(Collectors.toList()));
                return;
            }
            repoPath = repoService.getVirtualFileInfo(repoPath).getRepoPath();
            RepoPath finalRepoPath = repoPath;
            if (repoService.getLocalFederatedAndCachedRepoDescriptors().stream()
                    .anyMatch(e -> e.getKey().equals(finalRepoPath.getRepoKey()))) {
                actions.add(download());
            }
        } else if (RELEASE_BUNDLES_REPO.equals(repoType)) {
            actions.add(watch(repoPath));
            actions.add(download());
        } else if (SUPPORT_BUNDLES_REPO.equals(repoType)) {
            actions.add(download());
            actions.add(view(repoPath));
            actions.add(delete(repoPath));
        } else if (TRASH.equals(repoType)) {
            actions.add(restore());
            actions.add(deletePermanently());
        } else {
            actions.add(download());
            actions.add(view(repoPath));
            actions.add(copy(repoPath));
            actions.add(move(repoPath));
            actions.add(watch(repoPath));
            actions.add(delete(repoPath));
        }
        tabsAndActions.setActions(actions.stream()
                .filter(Objects::nonNull)
                .collect(Collectors.toList()));
    }

    TabOrAction downloadFolder(RepoPath repoPath) {
        FolderDownloadConfigDescriptor folderDownloadConfig = centralConfig.getDescriptor().getFolderDownloadConfig();
        if (authService.canRead(repoPath) && folderDownloadConfig.isEnabled() &&
                (!authService.isAnonymous() || folderDownloadConfig.isEnabledForAnonymous())) {
            return new TabOrAction("DownloadFolder");
        }
        return null;
    }

    private TabOrAction refresh() {
        return new TabOrAction("Refresh");
    }

    private TabOrAction favorites() {
        return new TabOrAction("Favorites");
    }

    private TabOrAction searchTrash() {
        return new TabOrAction("SearchTrash");
    }

    private TabOrAction emptyTrash() {
        return new TabOrAction("EmptyTrash");
    }

    private TabOrAction copyContent(RepoPath repoPath) {
        if (authService.canRead(repoPath) && authService.canDeployToLocalRepository()) {
            return new TabOrAction("CopyContent");
        }
        return null;
    }

    private TabOrAction moveContent(RepoPath repoPath) {
        if (authService.canDelete(repoPath) && authService.canDeployToLocalRepository()) {
            return new TabOrAction("MoveContent");
        }
        return null;
    }

    private TabOrAction watch(RepoPath repoPath) {
        if (userCanWatch(repoPath)) {
            if (isUserWatchingRepoPath(repoPath)) {
                return new TabOrAction("Unwatch");
            } else {
                return new TabOrAction("Watch");
            }
        }
        return null;
    }

    private TabOrAction zap(RepoPath repoPath) {
        LocalRepoDescriptor localRepoDescriptor = localOrCachedRepoDescriptor(repoPath);
        if (authService.canManage(repoPath) && localRepoDescriptor != null && localRepoDescriptor.isCache()) {
            return new TabOrAction("Zap");
        }
        return null;
    }

    private TabOrAction reindex(RepoPath repoPath, RepoType repoPkgType, String repoType) {
        final boolean hasReindex = ContextHelper.get().beanForType(PackageSpecification.class)
                .hasReindex(org.artifactory.common.repo.RepoType.LOCAL, repoPkgType);
        if (hasReindex && authService.canManage(repoPath) && "local".equals(repoType)) {
            return new TabOrAction("RecalculateIndex");
        }
        if (RepoType.Debian.equals(repoPkgType) && "cached".equals(repoType) && authService.canManage(repoPath)) {
            return new TabOrAction("CalculateDebianCoordinates");
        }
        return null;
    }

    private TabOrAction deleteVersions(RepoPath repoPath) {
        LocalRepoDescriptor localRepoDescriptor = localOrCachedRepoDescriptor(repoPath);
        if ((authService.canManage(repoPath) || authService.canDelete(repoPath))
                && authService.canRead(repoPath) && localRepoDescriptor != null && localRepoDescriptor.isLocal()) {
            return new TabOrAction("DeleteVersions");
        }
        return null;
    }

    private TabOrAction deleteContent(RepoPath repoPath) {
        if (authService.canDelete(repoPath)) {
            return new TabOrAction("DeleteContent");
        }
        return null;
    }

    private TabOrAction copy(RepoPath repoPath) {
        if (!NamingUtils.isSystem(repoPath.getPath())
                && !buildService.getAllBuildInfoRepos().contains(repoPath.getRepoKey())
                && authService.canRead(repoPath)
                && authService.canDeployToLocalRepository()) {
            return new TabOrAction("Copy");
        }
        return null;
    }

    private TabOrAction move(RepoPath repoPath) {
        if (!NamingUtils.isSystem(repoPath.getPath())
                && showMoveOnBuilds(repoPath)
                && authService.canDelete(repoPath)
                && authService.canDeployToLocalRepository()) {
            return new TabOrAction("Move");
        }
        return null;
    }

    private boolean showMoveOnBuilds(RepoPath repoPath) {
        return !(repoPath.getRepoKey()
                .equals(buildService.getBuildInfoRepoForDefaultProject()) &&
                buildService.getAllBuildInfoRepos().size() == 1);
    }

    private TabOrAction delete(RepoPath repoPath) {
        if (authService.canDelete(repoPath)) {
            return new TabOrAction("Delete");
        }
        return null;
    }

    private TabOrAction deletePermanently() {
        if (authService.isAdmin()) {
            return new TabOrAction("DeletePermanently");
        }
        return null;
    }

    private TabOrAction download() {
        return new TabOrAction("Download");
    }

    private TabOrAction view(RepoPath repoPath) {
        String path = repoPath.getPath();
        // PACKAGES is CRAN metadata index. it doesn't have an extension, so can't be added to the mimetypes
        if (NamingUtils.isViewable(path) || "class".equals(PathUtils.getExtension(path)) ||
                path.endsWith("/PACKAGES")) {
            return new TabOrAction("View");
        }
        return null;
    }

    private TabOrAction virtualZapCaches(String repoType) {
        if (authService.isAdmin() && VIRTUAL_REPO.equals(repoType)) {
            return new TabOrAction("ZapCaches");
        }
        return null;
    }

    private TabOrAction remoteVirtualReindex(RepoType repoPkgType, String repoType) {
        if (authService.isAdmin()) {
            if (packageSpecification.hasReindex(org.artifactory.common.repo.RepoType.REMOTE, repoPkgType) &&
                    REMOTE_REPO.equals(repoType)) {
                return new TabOrAction("RecalculateIndex");
            } else {
                boolean hasReindex =
                        packageSpecification.hasReindex(org.artifactory.common.repo.RepoType.VIRTUAL, repoPkgType);
                if (hasReindex && VIRTUAL_REPO.equals(repoType)) {
                    return new TabOrAction("RecalculateIndex");
                }
            }
        }
        return null;
    }

    private TabOrAction restore() {
        if (authService.isAdmin()) {
            return new TabOrAction("Restore");
        }
        return null;
    }

    private boolean isUserWatchingRepoPath(RepoPath repoPath) {
        ArtifactWatchAddon watchAddon = addonsManager.addonByType(ArtifactWatchAddon.class);
        return watchAddon.isUserWatchingRepo(repoPath, authService.currentUsername());
    }

    private boolean userCanWatch(RepoPath repoPath) {
        boolean isAddonSupported = addonsManager.isAddonSupported(AddonType.WATCH);
        return authService.canRead(repoPath) && isAddonSupported && !authService.isAnonymous()
                && !authService.isTransientUser() && !isThisBranchHasWatchAlready(repoPath);
    }

    private boolean isThisBranchHasWatchAlready(RepoPath repoPath) {
        ArtifactWatchAddon watchAddon = addonsManager.addonByType(ArtifactWatchAddon.class);
        Pair<RepoPath, WatchersInfo> nearestWatch =
                watchAddon.getNearestWatchDefinition(repoPath, authService.currentUsername());
        return nearestWatch != null && !(nearestWatch.getFirst().getPath().equals(repoPath.getPath()));
    }

    private LocalRepoDescriptor localOrCachedRepoDescriptor(RepoPath repoPath) {
        return repoService.localFederatedOrCachedRepoDescriptorByKey(repoPath.getRepoKey());
    }

    private boolean userCanDownloadFromRemote(TabsAndActions tabsAndActions) {
        String repoKey = tabsAndActions.getRepoPath().getRepoKey();

        if (repoKey.endsWith(RepoPath.REMOTE_CACHE_SUFFIX)) {
            return true;
        }

        RemoteRepoDescriptor descriptor = repoService.remoteRepoDescriptorByKey(repoKey);
        if (descriptor == null) {
            return false;
        }

        if (!descriptor.isStoreArtifactsLocally()) {
            return true;
        }

        RepoPath remoteCacheRepoPath = calcRepoPathFromRequestPath(
                descriptor.getKey() + RepoPath.REMOTE_CACHE_SUFFIX + RepoPath.PATH_SEPARATOR +
                        tabsAndActions.getRepoPath().getPath());
        if (authService.canDeploy(remoteCacheRepoPath) || repoService.exists(tabsAndActions.getRepoPath())) {
            return true;
        }

        tabsAndActions
                .setMessage("This item is not cached. You do not have a permission to download non-cached items.");
        return false;
    }
}
