package org.artifactory.rest.resource.artifact;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.rest.RestAddon;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.rest.constant.ArtifactRestConstants;
import org.artifactory.api.security.AuthorizationService;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static org.artifactory.api.rest.constant.ArtifactRestConstants.PATH_COPY;
import static org.artifactory.api.rest.constant.ArtifactRestConstants.PATH_FLAT;
import static org.artifactory.api.rest.constant.ArtifactRestConstants.PATH_MOVE;

/**
 * REST API used to copy an artifact from one path to another.
 *
 * @author Tomer Cohen
 */
@Component
@Scope(BeanDefinition.SCOPE_SINGLETON)
@Path(PATH_FLAT)
@RolesAllowed({AuthorizationService.ROLE_ADMIN, AuthorizationService.ROLE_USER})
public class FlatCopyMoveResource {

    /**
     * Move an item (file or folder) from one path to another, using Flat Move method.
     *
     * @param path     The source path.
     * @param target   The target path.
     * @param dryRun   Flag whether to perform a dry run before executing the actual move.
     * @param failFast Flag to indicate whether the operation should fail upon encountering an error.
     * @return The operation result
     * @throws Exception in case of error during move operation
     */
    @POST
    @Path(PATH_MOVE + "/{path: .+}")
    @Produces({ArtifactRestConstants.MT_COPY_MOVE_RESULT, MediaType.APPLICATION_JSON})
    public Response move(
            // The path of the source item to be moved/copied
            @PathParam("path") String path,
            // The target path to to move/copy the item.
            @QueryParam(ArtifactRestConstants.PARAM_TARGET) String target,
            // Flag to indicate whether to perform a dry run first. default false
            @QueryParam(ArtifactRestConstants.PARAM_DRY_RUN) int dryRun,
            //Flag to indicate whether the operation should fail upon encountering an error. default false
            @QueryParam(ArtifactRestConstants.PARAM_FAIL_FAST) int failFast) throws Exception {
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        return addonsManager.addonByType(RestAddon.class).moveFlat(path, target, dryRun, failFast);
    }

    /**
     * Copy an item (file or folder) from one path to another, using Flat Copy method.
     *
     * @param path     The source path.
     * @param target   The target path.
     * @param dryRun   Flag whether to perform a dry run before executing the actual copy.
     * @param failFast Flag to indicate whether the operation should fail upon encountering an error.
     * @return The operation result
     * @throws Exception in case of error during copy operation
     */
    @POST
    @Path(PATH_COPY + "/{path: .+}")
    @Produces({ArtifactRestConstants.MT_COPY_MOVE_RESULT, MediaType.APPLICATION_JSON})
    public Response copy(
            // The path of the source item to be moved/copied
            @PathParam("path") String path,
            // The target repository to to move/copy the item.
            @QueryParam(ArtifactRestConstants.PARAM_TARGET) String target,
            // Flag to indicate whether to perform a dry run first. default false
            @QueryParam(ArtifactRestConstants.PARAM_DRY_RUN) int dryRun,
            //Flag to indicate whether the operation should fail upon encountering an error. default false
            @QueryParam(ArtifactRestConstants.PARAM_FAIL_FAST) int failFast) throws Exception {
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        return addonsManager.addonByType(RestAddon.class).copyFlat(path, target, dryRun, failFast);
    }
}
